@@ -96,14 +96,9 @@ static GSList *no_reload_applets = NULL;
 static void panel_applet_frame_cnx_broken (ORBitConnection  *cnx,
 					   PanelAppletFrame *frame);
 
-static char *panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
-						   PanelWidget      *panel,
-						   const char       *iid,
-						   const char       *id);
-
-static void panel_applet_frame_activated (Bonobo_Unknown     object,
-					  CORBA_Environment *ev,
-					  gpointer          *data);
+static void panel_applet_frame_activated (CORBA_Object object,
+					  const char   *error_reason,
+					  gpointer     data);
 
 void
 panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
@@ -182,6 +177,7 @@ panel_applet_frame_set_flags_from_any (PanelAppletFrame *frame,
 {
 	gboolean major;
 	gboolean minor;
+	gboolean old_has_handle;
 	int      flags;
 	
 	flags = BONOBO_ARG_GET_SHORT (any);
@@ -192,7 +188,17 @@ panel_applet_frame_set_flags_from_any (PanelAppletFrame *frame,
 	panel_widget_set_applet_expandable (
 		frame->priv->panel, GTK_WIDGET (frame), major, minor);
 
+	old_has_handle = frame->priv->has_handle;
 	frame->priv->has_handle = (flags & APPLET_HAS_HANDLE) != 0;
+
+	if (!old_has_handle && frame->priv->has_handle) {
+		/* we've added an handle, so we need to get the background for
+		 * it */
+		PanelBackground *background;
+
+		background = &frame->priv->panel->background;
+		panel_applet_frame_change_background (frame, background->type);
+	}
 }
 
 static void
@@ -208,12 +214,12 @@ panel_applet_frame_set_size_hints_from_any (PanelAppletFrame *frame,
 
 	size_hints = g_new0 (int, seq->_length);
 
-	extra_size = 0;
-	if (frame->priv->has_handle)
+	if (frame->priv->has_handle) {
 		extra_size = HANDLE_SIZE + 1;
 	
-	for (i = 0; i < seq->_length; i++)
-		size_hints [i] = seq->_buffer [i] + extra_size;
+		for (i = 0; i < seq->_length; i++)
+			size_hints [i] = seq->_buffer [i] + extra_size;
+	}
 	
 	panel_widget_set_applet_size_hints (frame->priv->panel,
 					    GTK_WIDGET (frame),
@@ -324,7 +330,6 @@ panel_applet_frame_load (const gchar *iid,
 {
 	PanelAppletFrame           *frame;
 	CORBA_Environment           ev;
-	char                       *moniker;
 	PanelAppletFrameActivating *frame_act;
 
 	g_return_if_fail (iid != NULL);
@@ -349,15 +354,11 @@ panel_applet_frame_load (const gchar *iid,
 	frame_act->exactpos = exactpos;
 	frame_act->id       = g_strdup (id);
 
-	moniker = panel_applet_frame_construct_moniker (frame, panel, iid, id);
-
 	CORBA_exception_init (&ev);
 
-	bonobo_get_object_async (moniker, "IDL:Bonobo/Control:1.0", &ev,
-				 (BonoboMonikerAsyncFn) panel_applet_frame_activated,
-				 frame_act);
-
-	g_free (moniker);
+	bonobo_activation_activate_from_id_async (frame->priv->iid, 0,
+						  (BonoboActivationCallback) panel_applet_frame_activated,
+						  frame_act, &ev);
 }
 
 void
@@ -1048,7 +1049,7 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 				       frame->priv->iid);
 
 	dialog = gtk_message_dialog_new (NULL, 0,
-					 locked_down ? GTK_MESSAGE_INFO : GTK_MESSAGE_QUESTION,
+					 locked_down ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
 					 GTK_BUTTONS_NONE,
 					 problem_txt);
 	g_free (problem_txt);
@@ -1198,10 +1199,9 @@ panel_applet_frame_get_size_string (PanelAppletFrame *frame,
 }
 
 static char *
-panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
-				      PanelWidget      *panel,
-				      const char       *iid,
-				      const char       *id)
+panel_applet_frame_construct_item (PanelAppletFrame *frame,
+				   PanelWidget      *panel,
+				   const char       *id)
 {
 	char *retval;
 	char *bg_str;
@@ -1216,9 +1216,9 @@ panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
 	locked_down = panel_lockdown_get_locked_down ();
 
 	retval = g_strdup_printf (
-			"%s!prefs_key=/apps/panel/applets/%s/prefs;"
+			"prefs_key=/apps/panel/applets/%s/prefs;"
 			"background=%s;orient=%s;size=%s;locked_down=%s",
-			iid, id, bg_str,
+			id, bg_str,
 			panel_applet_frame_get_orient_string (frame, panel),
 			panel_applet_frame_get_size_string (frame, panel),
 			locked_down ? "true" : "false");
@@ -1243,40 +1243,82 @@ panel_applet_frame_event_listener (BonoboListener    *listener,
 }
 
 static void
-panel_applet_frame_activated (Bonobo_Unknown     object,
-			      CORBA_Environment *ev,
-			      gpointer          *data)
+panel_applet_frame_activated (CORBA_Object  object,
+			      const char   *error_reason,
+			      gpointer      data)
 {
 	PanelAppletFrameActivating *frame_act;
 	PanelAppletFrame   *frame;
 	GtkWidget          *widget;
 	BonoboControlFrame *control_frame;
 	Bonobo_Control      control;
+	Bonobo_ItemContainer container;
 	CORBA_Environment   corba_ev;
 	AppletInfo         *info;
 	char               *error;
+	char               *item_name;
 
 	frame_act = (PanelAppletFrameActivating *) data;
 	frame = frame_act->frame;
 
 	/* according to the source of bonobo control == NULL && no
 	   exception can happen, so handle it */
-	if (BONOBO_EX (ev) || object == CORBA_OBJECT_NIL) {
-		error = bonobo_exception_get_text (ev);
+	if (error_reason != NULL || object == CORBA_OBJECT_NIL) {
 		g_warning (G_STRLOC ": failed to load applet %s:\n%s",
-			   frame->priv->iid, error);
+			   frame->priv->iid, error_reason);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		g_free (frame_act->id);
 		g_free (frame_act);
+		return;
+	}
+
+	CORBA_exception_init (&corba_ev);
+
+	item_name = panel_applet_frame_construct_item (frame,
+						       frame->priv->panel,
+						       frame_act->id);
+
+	frame->priv->control = CORBA_OBJECT_NIL;
+	container = Bonobo_Unknown_queryInterface (object,
+						   "IDL:Bonobo/ItemContainer:1.0",
+						   &corba_ev);
+	if (!BONOBO_EX (&corba_ev) && container != CORBA_OBJECT_NIL) {
+		Bonobo_Unknown containee;
+
+		containee = Bonobo_ItemContainer_getObjectByName (container,
+								  item_name,
+								  TRUE,
+								  &corba_ev);
+		bonobo_object_release_unref (container, NULL);
+
+		if (!BONOBO_EX (&corba_ev) && containee != CORBA_OBJECT_NIL) {
+			frame->priv->control =
+				Bonobo_Unknown_queryInterface (containee,
+							       "IDL:Bonobo/Control:1.0",
+							       &corba_ev);
+
+			bonobo_object_release_unref (containee, NULL);
+		}
+	}
+	g_free (item_name);
+
+	if (frame->priv->control == CORBA_OBJECT_NIL) {
+		error = bonobo_exception_get_text (&corba_ev);
+		g_warning (G_STRLOC ": failed to get Bonobo/Control interface on applet %s:\n%s",
+                           frame->priv->iid, error);
+		panel_applet_frame_loading_failed (frame, frame_act->id);
+		CORBA_exception_free (&corba_ev);
+		bonobo_object_release_unref (object, NULL);
+                g_free (frame_act->id);
+		g_free (frame_act);
 		g_free (error);
 		return;
 	}
 
-	frame->priv->control = CORBA_Object_duplicate (object, NULL);
-
-	widget = bonobo_widget_new_control_from_objref (object,
+	widget = bonobo_widget_new_control_from_objref (frame->priv->control,
 							CORBA_OBJECT_NIL);
 
+	CORBA_exception_free (&corba_ev);
 	bonobo_object_release_unref (object, NULL);
 
 	if (!widget) {
